pause on
%GENERATE A NEURAL NETWORK
%net = network(numInputs,numLayers,biasConnect,inputConnect, layerConnect,outputConnect,targetConnect)
net = network(1,2,[1;0],[1; 0],[0 0; 1 0],[0 1],[0 1])

net.iw{1,1} %net.IW: numLayers-by-numInputs cell array of input weight values. 
net.iw{2,1}
net.b{1} %net.b: numLayers-by-1 cell array of bias values.

net.layers{1}.transferFcn = 'tansig';
net.layers{2}.transferFcn = 'logsig';

net.inputs{1}.range = [0 1; -1 1]; %change the number of elements in input 1 to 2, 
											  %by setting each element's range

pause

%CREATE A PERCEPTRON;
%net = newp(PR,S,TF,LF) takes these inputs,
%     PR - R x 2 matrix of min and max values for R input elements. 
%     S - Number of neurons. 
%     TF - Transfer function, default = 'hardlim'. 
%     LF - Learning function, default = 'learnp'. 

net = newp([0 1;0 1],1);

%SIMULATE A NEURAL NETWORK
%[Y,Pf,Af] = sim(net,P,Pi,Ai)
%[Y,Pf,Af] = sim(net,{Q TS},Pi,Ai)
%[Y,Pf,Af] = sim(net,Q,Pi,Ai)
%net - Network. 
%     P - Network inputs. 
%     Pi - Initial input delay conditions, default = zeros. 
%     Ai - Initial layer delay conditions, default = zeros. 
%Y - Network outputs. 
%     Pf - Final input delay conditions. 
%     Af - Final layer delay conditions. 
%Note that arguments Pi, Ai, Pf, and Af are optional and need 
%only be used for networks that have input or layer delays.

p1 = [.2; .9]; a1 = sim(net,p1)
p2 = [.2 .5 .1; .9 .3 .7]; a2 = sim(net,p2)
p3 = {[.2; .9] [.5; .3] [.1; .7]}; a3 = sim(net,p3)

pause

%CREATE A LINEAR LAYER
%net = newlin(PR,S,ID,LR)
%PR - R x 2 matrix of min and max values for R input elements. 
%     S - Number of elements in the output vector (Number of neurons). 
%     ID - Input delay vector, default = [0]. 
%     LR - Learning rate, default = 0.01.

net = newlin([0 2;0 2;0 2],2,[0 1]);
p1 = {[2; 0.5; 1] [1; 1.2; 0.1]};
[y1,pf] = sim(net,p1)

%TRAIN A NN
%[net,tr] = train(net,P,T,Pi,Ai)
%[net,tr] = train(net,P,T,Pi,Ai,VV,TV)
%train trains a network net according to net.trainFcn and net.trainParam.
%train(net,P,T,Pi,Ai) takes,
%     net - Network. 
%     P - Network inputs. 
%     T - Network targets, default = zeros. 
%     Pi - Initial input delay conditions, default = zeros. 
%     Ai - Initial layer delay conditions, default = zeros. 
%net - New network. 
%     TR - Training record (epoch and perf). 

%Generating a new neuron
net = newp([0 1;-2 2],1);
net.iw{1,1}
net.b{1}

%Training the perceptron alters its weight and bias values.
P = [0 1 0 1; 0 0 1 1];
T = [0 0 0 1];
net = train(net,P,T);
net.iw{1,1}
net.b{1}

pause

%INITIALIZE the NN
%The weights and biases are zeros again, which are the 
%initial values used by perceptron networks
net = init(net);
net.iw{1,1}
net.b{1}

pause

%The BACKPROPAGATION Algo
%Here input P and targets T define a simple function 
%which we can plot:

p = [0 1 2 3 4 5 6 7 8];
t = [0 0.84 0.91 0.14 -0.77 -0.96 -0.28 0.66 0.99];
plot(p,t,'o')

pause

%newff is used to create a two layer feed forward network
%The network will have an input (ranging from 0 to 8), 
%followed by a layer of 10 tansig neurons,
%followed by a layer with 1 purelin neuron. 
%trainlm backpropagation is used

net = newff([0 8],[10 1],{'tansig' 'purelin'},'trainlm');
y1 = sim(net,p)
plot(p,t,'o',p,y1,'x')

pause

%Here the network is trained for up to 50 epochs 
%to a error goal of 0.01, and then resimulated.

net.trainParam.epochs = 50;
net.trainParam.goal = 0.01;
net = train(net,p,t);
y2 = sim(net,p)
plot(p,t,'o',p,y1,'x',p,y2,'*')

pause

%ADAPT NN
%[net,Y,E,Pf,Af] = adapt(net,P,T,Pi,Ai)
%net - Network. 
%     P - Network inputs. 
%     T - Network targets, default = zeros. 
%     Pi - Initial input delay conditions, default = zeros. 
%     Ai - Initial layer delay conditions, default = zeros. 
%returns
%net - Updated network. 
%     Y   - Network outputs. 
%     E - Network errors. 
%     Pf - Final input delay conditions. 
%     Af - Final layer delay conditions.

%T1 is known to depend on P1
p1 = {-1  0 1 0 1 1 -1  0 -1 1 0 1};
t1 = {-1 -1 1 1 1 2  0 -1 -1 0 1 1};

%newlin is used to create a layer with an input range of [-1 1]),
%one neuron, input delays of 0 and 1, and a learning rate of 0.5
net = newlin([-1 1],1,[0 1],0.5);

pause

%Since this is the first call of adapt the default Pi is used
[net,y,e,pf] = adapt(net,p1,t1); %adapt the network
mse(e) %the network's mean squared error is displayed.
%Note the errors are quite large

pause

%using the previous Pf as the new initial delay conditions
p2 = {1 -1 -1 1 1 -1  0 0 0 1 -1 -1};
t2 = {2  0 -2 0 2  0 -1 0 0 1  0 -1};
[net,y,e,pf] = adapt(net,p2,t2,pf);
mse(e)

pause

%Here the network adapts for 100 passes through the entire sequence.
p3 = [p1 p2];
t3 = [t1 t2];
net.adaptParam.passes = 100;
[net,y,e] = adapt(net,p3,t3);
mse(e)